summaryrefslogtreecommitdiff
path: root/src/journal/journald.c
diff options
context:
space:
mode:
authorLennart Poettering <lennart@poettering.net>2012-07-01 20:17:48 +0200
committerLennart Poettering <lennart@poettering.net>2012-07-02 10:43:57 +0200
commite156d769c3ee756cdb20f8522ace9ea459a82655 (patch)
tree478af421d4a2b6274cc086a28c89a4471b1d1a99 /src/journal/journald.c
parent29252e9e5bad3b0bcfc45d9bc761aee4b0ece1da (diff)
journald: add Storage= setting to control where the journal is stored
Diffstat (limited to 'src/journal/journald.c')
-rw-r--r--src/journal/journald.c37
1 files changed, 30 insertions, 7 deletions
diff --git a/src/journal/journald.c b/src/journal/journald.c
index 7ea68aab3c..08597ae3eb 100644
--- a/src/journal/journald.c
+++ b/src/journal/journald.c
@@ -111,6 +111,16 @@ struct StdoutStream {
LIST_FIELDS(StdoutStream, stdout_stream);
};
+static const char* const storage_table[] = {
+ [STORAGE_AUTO] = "auto",
+ [STORAGE_VOLATILE] = "volatile",
+ [STORAGE_PERMANENT] = "permanent",
+ [STORAGE_NONE] = "none"
+};
+
+DEFINE_STRING_TABLE_LOOKUP(storage, Storage);
+DEFINE_CONFIG_PARSE_ENUM(config_parse_storage, storage, Storage, "Failed to parse storage setting");
+
static int server_flush_to_var(Server *s);
static uint64_t available_space(Server *s) {
@@ -397,7 +407,6 @@ static void server_vacuum(Server *s) {
free(p);
}
-
if (s->runtime_journal) {
if (asprintf(&p, "/run/log/journal/%s", ids) < 0) {
log_error("Out of memory.");
@@ -625,9 +634,7 @@ static void dispatch_message_real(
retry:
f = find_journal(s, realuid == 0 ? 0 : loginuid);
- if (!f)
- log_warning("Dropping message, as we can't find a place to store the data.");
- else {
+ if (f) {
r = journal_file_append_entry(f, NULL, iovec, n, &s->seqnum, NULL, NULL);
if ((r == -E2BIG || /* hit limit */
@@ -1955,12 +1962,23 @@ static int system_journal_open(Server *s) {
sd_id128_to_string(machine, ids);
- if (!s->system_journal) {
+ if (!s->system_journal &&
+ (s->storage == STORAGE_PERMANENT ||
+ s->storage == STORAGE_AUTO)) {
+
+ /* If in auto mode: first try to create the machine
+ * path, but not the prefix.
+ *
+ * If in permanent mode: create /var/log/journal and
+ * the machine path */
+
+ if (s->storage & STORAGE_PERMANENT)
+ (void) mkdir("/var/log/journal/", 0755);
- /* First try to create the machine path, but not the prefix */
fn = strappend("/var/log/journal/", ids);
if (!fn)
return -ENOMEM;
+
(void) mkdir(fn, 0755);
free(fn);
@@ -1988,7 +2006,8 @@ static int system_journal_open(Server *s) {
}
}
- if (!s->runtime_journal) {
+ if (!s->runtime_journal &&
+ (s->storage != STORAGE_NONE)) {
fn = join("/run/log/journal/", ids, "/system.journal", NULL);
if (!fn)
@@ -2048,6 +2067,10 @@ static int server_flush_to_var(Server *s) {
assert(s);
+ if (s->storage != STORAGE_AUTO &&
+ s->storage != STORAGE_PERMANENT)
+ return 0;
+
if (!s->runtime_journal)
return 0;